@@ -41,7 +41,7 @@ from castellan.i18n import _
 from castellan.key_manager import key_manager
 
 
-from barbicanclient import client as barbican_client
+from barbicanclient import client as barbican_client_import
 from barbicanclient import exceptions as barbican_exceptions
 from oslo_utils import timeutils
 from six.moves import urllib
@@ -120,7 +120,7 @@ class BarbicanKeyManager(key_manager.KeyManager):
                                    verify=self.conf.barbican.verify_ssl)
 
             self._barbican_endpoint = self._get_barbican_endpoint(auth, sess)
-            self._barbican_client = barbican_client.Client(
+            self._barbican_client = barbican_client_import.Client(
                 session=sess,
                 endpoint=self._barbican_endpoint)
             self._current_context = context
@@ -202,8 +202,12 @@ class BarbicanKeyManager(key_manager.KeyManager):
             latest_version = raw_data[-1]
             api_version = latest_version.get('id')
 
+        if endpoint[-1] != '/':
+            endpoint += '/'
+
         base_url = urllib.parse.urljoin(
             endpoint, api_version)
+
         return base_url
 
     def create_key(self, context, algorithm, length,
@@ -480,6 +484,11 @@ class BarbicanKeyManager(key_manager.KeyManager):
         else:
             secret_data = self._get_secret_data(secret)
 
+        if secret.secret_ref:
+            object_id = self._retrieve_secret_uuid(secret.secret_ref)
+        else:
+            object_id = None
+
         # convert created ISO8601 in Barbican to POSIX
         if secret.created:
             time_stamp = timeutils.parse_isotime(
@@ -491,11 +500,13 @@ class BarbicanKeyManager(key_manager.KeyManager):
                                secret.bit_length,
                                secret_data,
                                secret.name,
-                               created)
+                               created,
+                               object_id)
         else:
             return secret_type(secret_data,
                                secret.name,
-                               created)
+                               created,
+                               object_id)
 
     def _get_secret(self, context, object_id):
         """Returns the metadata of the secret.
@@ -574,3 +585,44 @@ class BarbicanKeyManager(key_manager.KeyManager):
                     uuid=managed_object_id)
             else:
                 raise exception.KeyManagerError(reason=e)
+
+    def list(self, context, object_type=None, metadata_only=False):
+        """Retrieves a list of managed objects that match the criteria.
+
+        If no search criteria is given, all objects are returned.
+
+        :param context: contains information of the user and the environment
+                     for the request (castellan/context.py)
+        :param object_type: the type of object to retrieve
+        :param metadata_only: whether secret data should be included
+        :raises KeyManagerError: if listing secrets fails
+        """
+        objects = []
+        barbican_client = self._get_barbican_client(context)
+
+        if object_type and object_type not in self._secret_type_dict:
+            msg = _("Invalid secret type: %s") % object_type
+            LOG.error(msg)
+            raise exception.KeyManagerError(reason=msg)
+
+        secret_type = self._secret_type_dict.get(object_type)
+
+        try:
+            secrets = barbican_client.secrets.list(secret_type=secret_type)
+        except (barbican_exceptions.HTTPAuthError,
+                barbican_exceptions.HTTPClientError,
+                barbican_exceptions.HTTPServerError) as e:
+            LOG.error(_("Error listing objects: %s"), e)
+            raise exception.KeyManagerError(reason=e)
+
+        for secret in secrets:
+            try:
+                obj = self._get_castellan_object(secret, metadata_only)
+                objects.append(obj)
+            except (barbican_exceptions.HTTPAuthError,
+                    barbican_exceptions.HTTPClientError,
+                    barbican_exceptions.HTTPServerError) as e:
+                LOG.warning(_("Error occurred while retrieving object "
+                              "metadata, not adding it to the list: %s"), e)
+
+        return objects
